/************************************************************************/
/*																		*/
/*	GPIO_demo.c	--	GPIO Demo Main Program								*/
/*																		*/
/************************************************************************/
/*	Author: Sam Bobrowicz												*/
/*	Copyright 2011, Digilent Inc.										*/
/************************************************************************/
/*  Module Description: 												*/
/*																		*/
/*	The GPIO Demo reference project demonstrates a simple usage of the	*/
/*	Nexys3's GPIO and UART in an EDK design. It utilizes a UART			*/
/*	controller, interrupt controller, timer, and 4 GPIO cores that are	*/
/*	connected to the Nexys3's LEDs, Switches, Push Buttons and Seven-	*/
/*	Segment display. The Timer is configured to trigger an interrupt	*/
/*	every second that increments the Seven-Segment display. The Push	*/
/*	buttons are configured to trigger an interrupt that causes a message*/
/*	to be sent across the UART, and that also causes a corresponding 	*/
/*  digit in the 7-Segment display to be blanked.						*/
/*																		*/
/*	This project demonstrates how to accomplish the following tasks in	*/
/*  an EDK design that targets a Nexys3:								*/
/*			*Using a UartLite core as stdout							*/
/*			*Connecting a GPIO core to an interrupt controller			*/
/*			*Connecting a Timer core to an interrupt controller			*/
/*			*Polling GPIO cores with simple memory access functions		*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/* 																		*/
/*		7/29/2011(SamB): Created										*/
/*		3/12/2013(SamB): Updated to work on AXI system					*/
/* 						 Updated to use svn_seg_axi core and document   */
/*						 it register map.								*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */

#include <stdio.h>
#include "xparameters.h"  //The hardware configuration describing constants
#include "xgpio.h"	//GPIO API functions
#include "xintc.h"	//Interrupt Controller API functions
#include "xil_io.h"	//Contains the Xil_Out32 and Xil_In32 functions
#include "mb_interface.h" //Contains the functions for registering the
						  //interrupt controller with the microblaze MP
#include "GPIO_demo.h"


/* ------------------------------------------------------------ */
/*				XPAR Constants									*/
/* ------------------------------------------------------------ */
/* All constants used from xparameters.h are located here so 	*/
/* that they may be easily accessed.							*/
/**/

#define LEDS_BASEADDR XPAR_LEDS_8BITS_BASEADDR
#define SS_BASEADDR XPAR_DIGILENT_SEVSEG_DISP_BASEADDR
#define SW_BASEADDR XPAR_DIP_SWITCHES_8BITS_BASEADDR
#define TMR_BASEADDR XPAR_TMRCTR_0_BASEADDR
#define BTNS_BASEADDR XPAR_PUSH_BUTTONS_4BITS_BASEADDR
#define BTNS_DEVICE_ID XPAR_PUSH_BUTTONS_4BITS_DEVICE_ID
#define INTC_DEVICE_ID XPAR_INTC_0_DEVICE_ID
#define BTNS_IRPT_ID XPAR_INTC_0_GPIO_2_VEC_ID
#define TMR_IRPT_ID XPAR_INTC_0_TMRCTR_0_VEC_ID

/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */

volatile uint ssVal;

/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */

int main()
{

	static XGpio pshBtns;
	static XIntc intCtrl;

	/*
     *Initialize the seven segment display to show zeros
     */
    ssVal = 0;
    Xil_Out32(SS_BASEADDR + bSsgCtrl, 0x00000000);
    Xil_Out32(SS_BASEADDR + bSsgHexData, 0x0000);

    /*
     *Initialize the driver structs for the Push button and interrupt cores.
     *This allows the API functions to be used with these cores.
     */
    XGpio_Initialize(&pshBtns, BTNS_DEVICE_ID);
    XIntc_Initialize(&intCtrl, INTC_DEVICE_ID);

    /*
     * Connect the function PushBtnHandler to the interrupt controller so that
     * it is called whenever the Push button GPIO core signals an interrupt.
     */
    XIntc_Connect(&intCtrl, BTNS_IRPT_ID, PushBtnHandler, &pshBtns);

    /*
     * Connect the function TmrHandler to the interrupt controller so that
     * it is called whenever the Timer core signals an interrupt.
     */
    XIntc_Connect(&intCtrl, TMR_IRPT_ID, TmrHandler, NULL);

    /*
     * Enable both interrupts at the interrupt controller
     */
    XIntc_Enable(&intCtrl, BTNS_IRPT_ID);
    XIntc_Enable(&intCtrl, TMR_IRPT_ID);

    /*
     * Register the interrupt controller with the microblaze
     * processor and then start the Interrupt controller so that it begins
     * listening to the push buttons and timer for triggers.
     */
    microblaze_register_handler(XIntc_DeviceInterruptHandler, INTC_DEVICE_ID);
    microblaze_enable_interrupts();
    XIntc_Start(&intCtrl, XIN_REAL_MODE);

    /*
     * Enable the push button GPIO core to begin sending interrupts to the
     * interrupt controller in response to changes in the button states
     */
    XGpio_InterruptEnable(&pshBtns, lBtnChannel);
    XGpio_InterruptGlobalEnable(&pshBtns);

    /*
     * Initialize and start the timer core so that it will trigger an interrupt
     * every second.
     */
    Xil_Out32(TMR_BASEADDR + bTmrTLR0, lTmrLoadVal);
    Xil_Out32(TMR_BASEADDR + bTmrTCSR0, bitTmrLoad);
    Xil_Out32(TMR_BASEADDR + bTmrTCSR0, fsTmrRun);

    /*
     * The UART core is connected as stdout, so the following sends a welcome
     * message to a connected terminal program.
     */
    xil_printf("\r\n----------------------------------------------------------\r\n");
    xil_printf("\r\nNexys3 Board GPIO DEMO \r\n");
    xil_printf("\r\n----------------------------------------------------------\r\n");

    /*
     * This while loop is never left during operation, unless a user reset is
     * triggered. It constantly polls the switches and sets the LEDs
     * accordingly. The GPIO API functions are not necessary when performing
     * simple read/write functions such as these: just use the Xil_Out/Xil_In
     * functions to write/read the contents of the data register located at the
     * GPIO core's Base address.
     */
    while(1)
    {
    	Xil_Out32(LEDS_BASEADDR, Xil_In32(SW_BASEADDR));
    }

    return 0;
}

/***	PushBtnHandler
**
**	Parameters:
**		CallBackRef - Pointer to the push button struct (pshBtns) initialized
**		in main.
**
**	Return Value:
**		None
**
**	Errors:
**		None
**
**	Description:
**		This function is connected to the interrupt handler such that it is
**		called whenever an interrupt is triggered by the push buttons. It
**		sends a message over the UART and also maps the push button state to
**		the seven-segment anodes.
*/
void PushBtnHandler(void *CallBackRef)
{
	XGpio *PushBtnPtr = (XGpio *)CallBackRef;
	u32 btnState = 0;

	/*
	 * Clear the push button interrupt at the GPIO core
	 */
	XGpio_InterruptClear(PushBtnPtr, lBtnChannel);

	xil_printf("Button Interrupt Triggered!!!\n\r");

	/*
	 * Write the current state of the buttons to the blank nibble of the Control
	 * register for the seven segment display controller
	 */
	btnState = XGpio_DiscreteRead(PushBtnPtr, lBtnChannel);
	btnState = btnState & mskSsgBlank;
	Xil_Out32(SS_BASEADDR + bSsgCtrl, btnState);
}


/***	TmrHandler
**
**	Parameters:
**		CallBackRef - NULL value passed through by the interrupt handler
**
**	Return Value:
**		None
**
**	Errors:
**		None
**
**	Description:
**		This function is connected to the interrupt handler such that it is
**		called whenever the timer triggers an interrupt. It increments the
**		number currently being displayed on the seven-segment.
*/
void TmrHandler(void *CallBackRef)
{
	u32 ssBuf;

	ssVal++;
	if (ssVal == 10)
	{
		ssVal = 0;
	}
	/*
	 * Writes the current value to each Anode's nibble within the HEX_DATA
	 * register of the seven segment display controller
	 */
	ssBuf = (ssVal) | (ssVal << 4) | (ssVal << 8) | (ssVal << 12);
	Xil_Out32(SS_BASEADDR + bSsgHexData, ssBuf);
	Xil_Out32(TMR_BASEADDR + bTmrTCSR0, fsTmrRun | bitTmrIntClr);
}


/* ------------------------------------------------------------ */

/************************************************************************/
